@@ -1,12 +1,26 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-/*
- * E-table-col.c: ETableCol implementation
+/* 
+ * e-table-col.c
+ * Copyright 1999, 2000, 2001, Ximian, Inc.
  *
- * Author:
- *   Miguel de Icaza (miguel@gnu.org)
+ * Authors:
+ *   Miguel de Icaza <miguel@ximian.com>
  *
- * (C) 1999 Ximian, Inc
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public
+ * License, version 2, as published by the Free Software Foundation.
+ *
+ * This library is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
+ *
+ * You should have received a copy of the GNU Library General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+ * 02111-1307, USA.
  */
+
 #include <config.h>
 #include <gtk/gtkobject.h>
 #include <gtk/gtksignal.h>
@@ -21,6 +35,7 @@ static GtkObjectClass *parent_class;
 enum {
 	ARG_0,
 	ARG_SORTABLE,
+	ARG_COMPARE_COL
 };
 
 static void
@@ -48,6 +63,9 @@ etc_set_arg (GtkObject *o, GtkArg *arg, guint arg_id)
 	case ARG_SORTABLE:
 		etc->sortable = GTK_VALUE_BOOL(*arg);
 		break;
+	case ARG_COMPARE_COL:
+		etc->compare_col = GTK_VALUE_INT(*arg);
+		break;
 	}
 }
 
@@ -60,6 +78,9 @@ etc_get_arg (GtkObject *o, GtkArg *arg, guint arg_id)
 	case ARG_SORTABLE:
 		GTK_VALUE_BOOL(*arg) = etc->sortable;
 		break;
+	case ARG_COMPARE_COL:
+		GTK_VALUE_INT(*arg) = etc->compare_col;
+		break;
 	default:
 		arg->type = GTK_TYPE_INVALID;
 		break;
@@ -76,6 +97,8 @@ e_table_col_class_init (GtkObjectClass *object_class)
 
 	gtk_object_add_arg_type ("ETableCol::sortable",
 				 GTK_TYPE_BOOL, GTK_ARG_READWRITE, ARG_SORTABLE);  
+	gtk_object_add_arg_type ("ETableCol::compare_col",
+				 GTK_TYPE_INT, GTK_ARG_READWRITE, ARG_COMPARE_COL);
 }
 
 static void
@@ -88,7 +111,7 @@ e_table_col_init (ETableCol *etc)
 	etc->priority = 0;
 }
 
-E_MAKE_TYPE(e_table_col, "ETableCol", ETableCol, e_table_col_class_init, e_table_col_init, PARENT_TYPE);
+E_MAKE_TYPE(e_table_col, "ETableCol", ETableCol, e_table_col_class_init, e_table_col_init, PARENT_TYPE)
 
 /** 
  * e_table_col_new:
@@ -119,7 +142,7 @@ E_MAKE_TYPE(e_table_col, "ETableCol", ETableCol, e_table_col_class_init, e_table
  */
 ETableCol *
 e_table_col_new (int col_idx, const char *text, double expansion, int min_width,
-		 ECell *ecell, GCompareFunc compare, gboolean resizable, int priority)
+		 ECell *ecell, GCompareFunc compare, gboolean resizable, gboolean disabled, int priority)
 {
 	ETableCol *etc;
 	
@@ -134,12 +157,14 @@ e_table_col_new (int col_idx, const char *text, double expansion, int min_width,
 	etc->is_pixbuf = FALSE;
 
 	etc->col_idx = col_idx;
+	etc->compare_col = col_idx;
 	etc->text = g_strdup (text);
 	etc->pixbuf = NULL;
 	etc->expansion = expansion;
 	etc->min_width = min_width;
 	etc->ecell = ecell;
 	etc->compare = compare;
+	etc->disabled = disabled;
 	etc->priority = priority;
 
 	etc->selected = 0;
@@ -178,7 +203,7 @@ e_table_col_new (int col_idx, const char *text, double expansion, int min_width,
  */
 ETableCol *
 e_table_col_new_with_pixbuf (int col_idx, const char *text, GdkPixbuf *pixbuf, double expansion, int min_width,
-			     ECell *ecell, GCompareFunc compare, gboolean resizable, int priority)
+			     ECell *ecell, GCompareFunc compare, gboolean resizable, gboolean disabled, int priority)
 {
 	ETableCol *etc;
 	
@@ -193,12 +218,14 @@ e_table_col_new_with_pixbuf (int col_idx, const char *text, GdkPixbuf *pixbuf, d
 	etc->is_pixbuf = TRUE;
 
 	etc->col_idx = col_idx;
+	etc->compare_col = col_idx;
 	etc->text = g_strdup(text);
 	etc->pixbuf = pixbuf;
 	etc->expansion = expansion;
 	etc->min_width = min_width;
 	etc->ecell = ecell;
 	etc->compare = compare;
+	etc->disabled = disabled;
 	etc->priority = priority;
 
 	etc->selected = 0;
