@@ -47,7 +47,7 @@ enum {
 	LAST_SIGNAL
 };
 
-static gint etg_signals [LAST_SIGNAL] = { 0, };
+static guint etg_signals [LAST_SIGNAL] = { 0, };
 
 static gboolean etg_get_focus (ETableGroup      *etg);
 static void etg_destroy (GtkObject *object);
@@ -387,10 +387,10 @@ e_table_group_compute_location (ETableGroup *etg, int *x, int *y, int *row, int
  * @row: A pointer to the row number to find.
  * @col: A pointer to the col number to find.
  *
- * This routine finds the view cell (row, col) in the %ETableGroup.
- * If that location is in the %ETableGroup *x and *y are set to the
+ * This routine finds the view cell (row, col) in the #ETableGroup.
+ * If that location is in the #ETableGroup *@x and *@y are set to the
  * upper left hand corner of the cell found.  If that location is not
- * in the %ETableGroup, the number of rows in the %ETableGroup is
+ * in the #ETableGroup, the number of rows in the #ETableGroup is
  * removed from the value row points to.
  */
 void
@@ -699,4 +699,4 @@ etg_class_init (GtkObjectClass *object_class)
 	E_OBJECT_CLASS_ADD_SIGNALS (object_class, etg_signals, LAST_SIGNAL);
 }
 
-E_MAKE_TYPE (e_table_group, "ETableGroup", ETableGroup, etg_class_init, NULL, PARENT_TYPE);
+E_MAKE_TYPE (e_table_group, "ETableGroup", ETableGroup, etg_class_init, NULL, PARENT_TYPE)
